@@ -1014,30 +1014,30 @@ END_TEST
 
 TEST_COLLECTION(path)
 {
-	litest_add_no_device("path:create", path_create_NULL);
-	litest_add_no_device("path:create", path_create_invalid);
-	litest_add_no_device("path:create", path_create_invalid_file);
-	litest_add_no_device("path:create", path_create_invalid_kerneldev);
-	litest_add_no_device("path:create", path_create_pathmax_file);
-	litest_add_no_device("path:create", path_create_destroy);
-	litest_add("path:create", path_force_destroy, LITEST_ANY, LITEST_ANY);
-	litest_add_no_device("path:create", path_set_user_data);
-	litest_add_no_device("path:suspend", path_suspend);
-	litest_add_no_device("path:suspend", path_double_suspend);
-	litest_add_no_device("path:suspend", path_double_resume);
-	litest_add_no_device("path:suspend", path_add_device_suspend_resume);
-	litest_add_no_device("path:suspend", path_add_device_suspend_resume_fail);
-	litest_add_no_device("path:suspend", path_add_device_suspend_resume_remove_device);
-	litest_add_for_device("path:seat", path_added_seat, LITEST_SYNAPTICS_CLICKPAD_X220);
-	litest_add_for_device("path:seat", path_seat_change, LITEST_SYNAPTICS_CLICKPAD_X220);
-	litest_add("path:device events", path_added_device, LITEST_ANY, LITEST_ANY);
-	litest_add("path:device events", path_device_sysname, LITEST_ANY, LITEST_ANY);
-	litest_add_for_device("path:device events", path_add_device, LITEST_SYNAPTICS_CLICKPAD_X220);
-	litest_add_no_device("path:device events", path_add_invalid_path);
-	litest_add_for_device("path:device events", path_remove_device, LITEST_SYNAPTICS_CLICKPAD_X220);
-	litest_add_for_device("path:device events", path_double_remove_device, LITEST_SYNAPTICS_CLICKPAD_X220);
-	litest_add_no_device("path:seat", path_seat_recycle);
-	litest_add_for_device("path:udev", path_udev_assign_seat, LITEST_SYNAPTICS_CLICKPAD_X220);
+	litest_add_no_device(path_create_NULL);
+	litest_add_no_device(path_create_invalid);
+	litest_add_no_device(path_create_invalid_file);
+	litest_add_no_device(path_create_invalid_kerneldev);
+	litest_add_no_device(path_create_pathmax_file);
+	litest_add_no_device(path_create_destroy);
+	litest_add(path_force_destroy, LITEST_ANY, LITEST_ANY);
+	litest_add_no_device(path_set_user_data);
+	litest_add_no_device(path_suspend);
+	litest_add_no_device(path_double_suspend);
+	litest_add_no_device(path_double_resume);
+	litest_add_no_device(path_add_device_suspend_resume);
+	litest_add_no_device(path_add_device_suspend_resume_fail);
+	litest_add_no_device(path_add_device_suspend_resume_remove_device);
+	litest_add_for_device(path_added_seat, LITEST_SYNAPTICS_CLICKPAD_X220);
+	litest_add_for_device(path_seat_change, LITEST_SYNAPTICS_CLICKPAD_X220);
+	litest_add(path_added_device, LITEST_ANY, LITEST_ANY);
+	litest_add(path_device_sysname, LITEST_ANY, LITEST_ANY);
+	litest_add_for_device(path_add_device, LITEST_SYNAPTICS_CLICKPAD_X220);
+	litest_add_no_device(path_add_invalid_path);
+	litest_add_for_device(path_remove_device, LITEST_SYNAPTICS_CLICKPAD_X220);
+	litest_add_for_device(path_double_remove_device, LITEST_SYNAPTICS_CLICKPAD_X220);
+	litest_add_no_device(path_seat_recycle);
+	litest_add_for_device(path_udev_assign_seat, LITEST_SYNAPTICS_CLICKPAD_X220);
 
-	litest_add_no_device("path:ignore", path_ignore_device);
+	litest_add_no_device(path_ignore_device);
 }
