@@ -312,7 +312,7 @@ START_TEST(pointer_absolute_initial_state)
 		libinput_event_destroy(ev2);
 	}
 
-	libinput_unref(libinput2);
+	litest_destroy_context(libinput2);
 }
 END_TEST
 
@@ -490,7 +490,7 @@ START_TEST(pointer_button_auto_release)
 		ck_assert_int_eq(buttons[i].released, 1);
 	}
 
-	libinput_unref(libinput);
+	litest_destroy_context(libinput);
 }
 END_TEST
 
@@ -602,33 +602,6 @@ out:
 	return angle;
 }
 
-static enum libinput_pointer_axis_source
-wheel_source(struct litest_device *dev, int which)
-{
-	struct udev_device *d;
-	bool is_tilt = false;
-
-	d = libinput_device_get_udev_device(dev->libinput_device);
-	litest_assert_ptr_notnull(d);
-
-	switch(which) {
-	case REL_WHEEL:
-		is_tilt = !!udev_device_get_property_value(d, "MOUSE_WHEEL_TILT_VERTICAL");
-		break;
-	case REL_HWHEEL:
-		is_tilt = !!udev_device_get_property_value(d, "MOUSE_WHEEL_TILT_HORIZONTAL");
-		break;
-	default:
-		litest_abort_msg("Invalid source axis %d\n", which);
-		break;
-	}
-
-	udev_device_unref(d);
-	return is_tilt ?
-		LIBINPUT_POINTER_AXIS_SOURCE_WHEEL_TILT :
-		LIBINPUT_POINTER_AXIS_SOURCE_WHEEL;
-}
-
 static void
 test_wheel_event(struct litest_device *dev, int which, int amount)
 {
@@ -641,7 +614,7 @@ test_wheel_event(struct litest_device *dev, int which, int amount)
 	double scroll_step, expected, discrete;
 
 	scroll_step = wheel_click_angle(dev, which);
-	source = wheel_source(dev, which);
+	source = LIBINPUT_POINTER_AXIS_SOURCE_WHEEL;
 	expected = amount * scroll_step;
 	discrete = amount;
 
@@ -888,7 +861,7 @@ START_TEST(pointer_seat_button_count)
 
 	for (i = 0; i < num_devices; ++i)
 		litest_delete_device(devices[i]);
-	libinput_unref(libinput);
+	litest_destroy_context(libinput);
 }
 END_TEST
 
@@ -1171,8 +1144,8 @@ START_TEST(pointer_scroll_button_middle_emulation)
 
 	litest_drain_events(li);
 
-	litest_button_click_debounced(dev, li, BTN_LEFT, 1);
-	litest_button_click_debounced(dev, li, BTN_RIGHT, 1);
+	litest_button_click(dev, BTN_LEFT, 1);
+	litest_button_click(dev, BTN_RIGHT, 1);
 	libinput_dispatch(li);
 	litest_timeout_buttonscroll();
 	libinput_dispatch(li);
@@ -1184,8 +1157,8 @@ START_TEST(pointer_scroll_button_middle_emulation)
 
 	libinput_dispatch(li);
 
-	litest_button_click_debounced(dev, li, BTN_LEFT, 0);
-	litest_button_click_debounced(dev, li, BTN_RIGHT, 0);
+	litest_button_click(dev, BTN_LEFT, 0);
+	litest_button_click(dev, BTN_RIGHT, 0);
 	libinput_dispatch(li);
 
 	litest_assert_scroll(li, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL, -1);
@@ -1223,7 +1196,504 @@ START_TEST(pointer_scroll_button_device_remove_while_down)
 	litest_delete_device(dev);
 	libinput_dispatch(li);
 
-	libinput_unref(li);
+	litest_destroy_context(li);
+}
+END_TEST
+
+static void
+litest_enable_scroll_button_lock(struct litest_device *dev,
+				 unsigned int button)
+{
+	struct libinput_device *device = dev->libinput_device;
+	enum libinput_config_status status;
+
+	status = libinput_device_config_scroll_set_method(device,
+							  LIBINPUT_CONFIG_SCROLL_ON_BUTTON_DOWN);
+	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_SUCCESS);
+
+	status = libinput_device_config_scroll_set_button(device, button);
+	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_SUCCESS);
+
+	status = libinput_device_config_scroll_set_button_lock(device,
+							       LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_ENABLED);
+	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_SUCCESS);
+}
+
+START_TEST(pointer_scroll_button_lock)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	litest_enable_scroll_button_lock(dev, BTN_LEFT);
+	litest_disable_middleemu(dev);
+
+	litest_drain_events(li);
+
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+
+	litest_assert_empty_queue(li);
+
+	litest_timeout_buttonscroll();
+	libinput_dispatch(li);
+
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+
+	libinput_dispatch(li);
+
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+	libinput_dispatch(li);
+
+	litest_assert_scroll(li, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL, 6);
+
+	litest_assert_empty_queue(li);
+
+	/* back to motion */
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+}
+END_TEST
+
+START_TEST(pointer_scroll_button_lock_defaults)
+{
+	struct litest_device *dev = litest_current_device();
+	enum libinput_config_scroll_button_lock_state state;
+
+	state = libinput_device_config_scroll_get_button_lock(dev->libinput_device);
+	ck_assert_int_eq(state, LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_DISABLED);
+	state = libinput_device_config_scroll_get_default_button_lock(dev->libinput_device);
+	ck_assert_int_eq(state, LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_DISABLED);
+}
+END_TEST
+
+START_TEST(pointer_scroll_button_lock_config)
+{
+	struct litest_device *dev = litest_current_device();
+	enum libinput_config_status status;
+	enum libinput_config_scroll_button_lock_state state;
+
+	state = libinput_device_config_scroll_get_button_lock(dev->libinput_device);
+	ck_assert_int_eq(state, LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_DISABLED);
+	state = libinput_device_config_scroll_get_default_button_lock(dev->libinput_device);
+	ck_assert_int_eq(state, LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_DISABLED);
+
+	status = libinput_device_config_scroll_set_button_lock(dev->libinput_device,
+							       LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_DISABLED);
+	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_SUCCESS);
+	state = libinput_device_config_scroll_get_button_lock(dev->libinput_device);
+	ck_assert_int_eq(state, LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_DISABLED);
+
+
+	status = libinput_device_config_scroll_set_button_lock(dev->libinput_device,
+							       LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_ENABLED);
+	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_SUCCESS);
+	state = libinput_device_config_scroll_get_button_lock(dev->libinput_device);
+	ck_assert_int_eq(state, LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_ENABLED);
+
+	status = libinput_device_config_scroll_set_button_lock(dev->libinput_device,
+							       LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_ENABLED + 1);
+	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_INVALID);
+}
+END_TEST
+
+START_TEST(pointer_scroll_button_lock_enable_while_down)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	litest_disable_middleemu(dev);
+	litest_drain_events(li);
+
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+
+	/* Enable lock while button is down */
+	litest_enable_scroll_button_lock(dev, BTN_LEFT);
+
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_empty_queue(li);
+
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+	litest_assert_empty_queue(li);
+
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+
+	/* no scrolling yet */
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+
+	/* but on the next button press we scroll lock */
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+	libinput_dispatch(li);
+	litest_timeout_buttonscroll();
+	libinput_dispatch(li);
+
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+
+	litest_assert_scroll(li, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL, 6);
+
+	litest_assert_empty_queue(li);
+
+	/* back to motion */
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+}
+END_TEST
+
+START_TEST(pointer_scroll_button_lock_enable_while_down_just_lock)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	litest_disable_middleemu(dev);
+	litest_drain_events(li);
+
+	/* switch method first, but enable lock when we already have a
+	 * button down */
+	libinput_device_config_scroll_set_method(dev->libinput_device,
+					LIBINPUT_CONFIG_SCROLL_ON_BUTTON_DOWN);
+	libinput_device_config_scroll_set_button(dev->libinput_device,
+					BTN_LEFT);
+
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	libinput_device_config_scroll_set_button_lock(dev->libinput_device,
+					LIBINPUT_CONFIG_SCROLL_BUTTON_LOCK_ENABLED);
+
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+	litest_assert_empty_queue(li);
+
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+
+	/* no scrolling yet */
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+
+	/* but on the next button press we scroll lock */
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+	libinput_dispatch(li);
+	litest_timeout_buttonscroll();
+	libinput_dispatch(li);
+
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+
+	litest_assert_scroll(li, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL, 6);
+
+	litest_assert_empty_queue(li);
+
+	/* back to motion */
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+}
+END_TEST
+
+START_TEST(pointer_scroll_button_lock_otherbutton)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	litest_disable_middleemu(dev);
+	litest_drain_events(li);
+
+	litest_enable_scroll_button_lock(dev, BTN_LEFT);
+
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+	litest_assert_empty_queue(li);
+	litest_timeout_buttonscroll();
+	libinput_dispatch(li);
+
+	/* other button passes on normally */
+	litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+	litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_RELEASED);
+	litest_assert_empty_queue(li);
+
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_AXIS);
+
+	/* other button passes on normally */
+	litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+	litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_RELEASED);
+
+	/* stop scroll lock */
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_AXIS);
+
+	/* other button passes on normally */
+	litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+	litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_RELEASED);
+
+	litest_assert_empty_queue(li);
+}
+END_TEST
+
+START_TEST(pointer_scroll_button_lock_enable_while_otherbutton_down)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	litest_disable_middleemu(dev);
+	litest_drain_events(li);
+
+	litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+	litest_timeout_middlebutton();
+	litest_drain_events(li);
+
+	/* Enable lock while button is down */
+	litest_enable_scroll_button_lock(dev, BTN_LEFT);
+
+	/* We only enable once we go to a neutral state so this still counts
+	 * as normal button event */
+	for (int twice = 0; twice < 2; twice++) {
+		litest_button_click_debounced(dev, li, BTN_LEFT, true);
+		litest_button_click_debounced(dev, li, BTN_LEFT, false);
+		litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+		litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+
+		for (int i = 0; i < 10; i++) {
+			litest_event(dev, EV_REL, REL_X, 1);
+			litest_event(dev, EV_REL, REL_Y, 6);
+			litest_event(dev, EV_SYN, SYN_REPORT, 0);
+		}
+		litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+	}
+
+	litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_RELEASED);
+	litest_assert_empty_queue(li);
+
+	/* now we should trigger it */
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+	litest_timeout_buttonscroll();
+	litest_assert_empty_queue(li);
+
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+	litest_assert_scroll(li, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL, 6);
+	litest_assert_empty_queue(li);
+
+	/* back to motion */
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+	litest_assert_only_typed_events(li, LIBINPUT_EVENT_POINTER_MOTION);
+}
+END_TEST
+
+enum mb_buttonorder {
+	LLRR, /* left down, left up, r down, r up */
+	LRLR, /* left down, right down, left up, right up */
+	LRRL,
+	RRLL,
+	RLRL,
+	RLLR,
+	_MB_BUTTONORDER_COUNT
+};
+
+START_TEST(pointer_scroll_button_lock_middlebutton)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+	enum mb_buttonorder buttonorder = _i; /* ranged test */
+
+	if (!libinput_device_config_middle_emulation_is_available(dev->libinput_device))
+		return;
+
+	litest_enable_middleemu(dev);
+
+	litest_enable_scroll_button_lock(dev, BTN_LEFT);
+	litest_drain_events(li);
+
+	/* We expect scroll lock to work only where left and right are never
+	 * held down simultaneously. Everywhere else we expect middle button
+	 * instead.
+	 */
+	switch (buttonorder) {
+	case LLRR:
+		litest_button_click_debounced(dev, li, BTN_LEFT, true);
+		litest_button_click_debounced(dev, li, BTN_LEFT, false);
+		litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+		litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+		break;
+	case LRLR:
+		litest_button_click_debounced(dev, li, BTN_LEFT, true);
+		litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+		litest_button_click_debounced(dev, li, BTN_LEFT, false);
+		litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+		break;
+	case LRRL:
+		litest_button_click_debounced(dev, li, BTN_LEFT, true);
+		litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+		litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+		litest_button_click_debounced(dev, li, BTN_LEFT, false);
+		break;
+	case RRLL:
+		litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+		litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+		litest_button_click_debounced(dev, li, BTN_LEFT, true);
+		litest_button_click_debounced(dev, li, BTN_LEFT, false);
+		break;
+	case RLRL:
+		litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+		litest_button_click_debounced(dev, li, BTN_LEFT, true);
+		litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+		litest_button_click_debounced(dev, li, BTN_LEFT, false);
+		break;
+	case RLLR:
+		litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+		litest_button_click_debounced(dev, li, BTN_LEFT, true);
+		litest_button_click_debounced(dev, li, BTN_LEFT, false);
+		litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+		break;
+	default:
+		abort();
+	}
+
+	libinput_dispatch(li);
+	litest_timeout_middlebutton();
+	litest_timeout_buttonscroll();
+	libinput_dispatch(li);
+
+	/* motion events are the same for all of them */
+	for (int i = 0; i < 10; i++) {
+		litest_event(dev, EV_REL, REL_X, 1);
+		litest_event(dev, EV_REL, REL_Y, 6);
+		litest_event(dev, EV_SYN, SYN_REPORT, 0);
+	}
+
+	libinput_dispatch(li);
+
+	switch (buttonorder) {
+	case LLRR:
+	case RRLL:
+		litest_button_click_debounced(dev, li, BTN_LEFT, true);
+		litest_button_click_debounced(dev, li, BTN_LEFT, false);
+		break;
+	default:
+		break;
+	}
+
+	libinput_dispatch(li);
+
+	switch (buttonorder) {
+	case LLRR:
+	case RRLL:
+		litest_assert_button_event(li, BTN_RIGHT,
+					   LIBINPUT_BUTTON_STATE_PRESSED);
+		litest_assert_button_event(li, BTN_RIGHT,
+					   LIBINPUT_BUTTON_STATE_RELEASED);
+		litest_assert_scroll(li, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL, 6);
+		litest_assert_empty_queue(li);
+		break;
+	case LRLR:
+	case LRRL:
+	case RLRL:
+	case RLLR:
+		litest_assert_button_event(li, BTN_MIDDLE,
+					   LIBINPUT_BUTTON_STATE_PRESSED);
+		litest_assert_button_event(li, BTN_MIDDLE,
+					   LIBINPUT_BUTTON_STATE_RELEASED);
+		litest_assert_only_typed_events(li,
+						LIBINPUT_EVENT_POINTER_MOTION);
+		break;
+	default:
+		abort();
+	}
+
+}
+END_TEST
+
+START_TEST(pointer_scroll_button_lock_doubleclick_nomove)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput *li = dev->libinput;
+
+	litest_disable_middleemu(dev);
+	litest_enable_scroll_button_lock(dev, BTN_LEFT);
+	litest_drain_events(li);
+
+	/* double click without move in between counts as single click */
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+	litest_assert_empty_queue(li);
+	litest_button_click_debounced(dev, li, BTN_LEFT, true);
+	litest_button_click_debounced(dev, li, BTN_LEFT, false);
+
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_button_event(li, BTN_LEFT, LIBINPUT_BUTTON_STATE_RELEASED);
+	litest_assert_empty_queue(li);
+
+	/* But a non-scroll button it should work normally */
+	litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+	litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+	litest_button_click_debounced(dev, li, BTN_RIGHT, true);
+	litest_button_click_debounced(dev, li, BTN_RIGHT, false);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_RELEASED);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_PRESSED);
+	litest_assert_button_event(li, BTN_RIGHT, LIBINPUT_BUTTON_STATE_RELEASED);
+	litest_assert_empty_queue(li);
+
 }
 END_TEST
 
@@ -1447,39 +1917,6 @@ START_TEST(pointer_accel_profile_defaults)
 }
 END_TEST
 
-START_TEST(pointer_accel_profile_defaults_noprofile)
-{
-	struct litest_device *dev = litest_current_device();
-	struct libinput_device *device = dev->libinput_device;
-	enum libinput_config_status status;
-	enum libinput_config_accel_profile profile;
-	uint32_t profiles;
-
-	ck_assert(libinput_device_config_accel_is_available(device));
-
-	profile = libinput_device_config_accel_get_default_profile(device);
-	ck_assert_int_eq(profile, LIBINPUT_CONFIG_ACCEL_PROFILE_NONE);
-
-	profile = libinput_device_config_accel_get_profile(device);
-	ck_assert_int_eq(profile, LIBINPUT_CONFIG_ACCEL_PROFILE_NONE);
-
-	profiles = libinput_device_config_accel_get_profiles(device);
-	ck_assert_int_eq(profiles, LIBINPUT_CONFIG_ACCEL_PROFILE_NONE);
-
-	status = libinput_device_config_accel_set_profile(device,
-							  LIBINPUT_CONFIG_ACCEL_PROFILE_FLAT);
-	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_UNSUPPORTED);
-	profile = libinput_device_config_accel_get_profile(device);
-	ck_assert_int_eq(profile, LIBINPUT_CONFIG_ACCEL_PROFILE_NONE);
-
-	status = libinput_device_config_accel_set_profile(device,
-							  LIBINPUT_CONFIG_ACCEL_PROFILE_ADAPTIVE);
-	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_UNSUPPORTED);
-	profile = libinput_device_config_accel_get_profile(device);
-	ck_assert_int_eq(profile, LIBINPUT_CONFIG_ACCEL_PROFILE_NONE);
-}
-END_TEST
-
 START_TEST(pointer_accel_profile_invalid)
 {
 	struct litest_device *dev = litest_current_device();
@@ -2657,7 +3094,7 @@ START_TEST(debounce_remove_device_button_up)
 	litest_timeout_debounce();
 	libinput_dispatch(li);
 
-	libinput_unref(li);
+	litest_destroy_context(li);
 }
 END_TEST
 
@@ -2682,7 +3119,7 @@ START_TEST(debounce_remove_device_button_down)
 	litest_timeout_debounce();
 	libinput_dispatch(li);
 
-	libinput_unref(li);
+	litest_destroy_context(li);
 }
 END_TEST
 
@@ -2691,78 +3128,90 @@ TEST_COLLECTION(pointer)
 	struct range axis_range = {ABS_X, ABS_Y + 1};
 	struct range compass = {0, 7}; /* cardinal directions */
 	struct range buttons = {BTN_LEFT, BTN_TASK + 1};
+	struct range buttonorder = {0, _MB_BUTTONORDER_COUNT};
 
-	litest_add("pointer:motion", pointer_motion_relative, LITEST_RELATIVE, LITEST_POINTINGSTICK);
-	litest_add_for_device("pointer:motion", pointer_motion_relative_zero, LITEST_MOUSE);
-	litest_add_ranged("pointer:motion", pointer_motion_relative_min_decel, LITEST_RELATIVE, LITEST_POINTINGSTICK, &compass);
-	litest_add("pointer:motion", pointer_motion_absolute, LITEST_ABSOLUTE, LITEST_ANY);
-	litest_add("pointer:motion", pointer_motion_unaccel, LITEST_RELATIVE, LITEST_ANY);
-	litest_add("pointer:button", pointer_button, LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add_no_device("pointer:button", pointer_button_auto_release);
-	litest_add_no_device("pointer:button", pointer_seat_button_count);
-	litest_add_for_device("pointer:button", pointer_button_has_no_button, LITEST_KEYBOARD);
-	litest_add("pointer:button", pointer_recover_from_lost_button_count, LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add("pointer:scroll", pointer_scroll_wheel, LITEST_WHEEL, LITEST_TABLET);
-	litest_add("pointer:scroll", pointer_scroll_button, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
-	litest_add("pointer:scroll", pointer_scroll_button_noscroll, LITEST_ABSOLUTE|LITEST_BUTTON, LITEST_RELATIVE);
-	litest_add("pointer:scroll", pointer_scroll_button_noscroll, LITEST_ANY, LITEST_RELATIVE|LITEST_BUTTON);
-	litest_add("pointer:scroll", pointer_scroll_button_no_event_before_timeout, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
-	litest_add("pointer:scroll", pointer_scroll_button_middle_emulation, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
-	litest_add("pointer:scroll", pointer_scroll_button_device_remove_while_down, LITEST_ANY, LITEST_RELATIVE|LITEST_BUTTON);
-	litest_add("pointer:scroll", pointer_scroll_nowheel_defaults, LITEST_RELATIVE|LITEST_BUTTON, LITEST_WHEEL);
-	litest_add_for_device("pointer:scroll", pointer_scroll_defaults_logitech_marble , LITEST_LOGITECH_TRACKBALL);
-	litest_add("pointer:scroll", pointer_scroll_natural_defaults, LITEST_WHEEL, LITEST_TABLET);
-	litest_add("pointer:scroll", pointer_scroll_natural_defaults_noscroll, LITEST_ANY, LITEST_WHEEL);
-	litest_add("pointer:scroll", pointer_scroll_natural_enable_config, LITEST_WHEEL, LITEST_TABLET);
-	litest_add("pointer:scroll", pointer_scroll_natural_wheel, LITEST_WHEEL, LITEST_TABLET);
-	litest_add("pointer:scroll", pointer_scroll_has_axis_invalid, LITEST_WHEEL, LITEST_TABLET);
+	litest_add(pointer_motion_relative, LITEST_RELATIVE, LITEST_POINTINGSTICK);
+	litest_add_for_device(pointer_motion_relative_zero, LITEST_MOUSE);
+	litest_add_ranged(pointer_motion_relative_min_decel, LITEST_RELATIVE, LITEST_POINTINGSTICK, &compass);
+	litest_add(pointer_motion_absolute, LITEST_ABSOLUTE, LITEST_ANY);
+	litest_add(pointer_motion_unaccel, LITEST_RELATIVE, LITEST_ANY);
+	litest_add(pointer_button, LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add_no_device(pointer_button_auto_release);
+	litest_add_no_device(pointer_seat_button_count);
+	litest_add_for_device(pointer_button_has_no_button, LITEST_KEYBOARD);
+	litest_add(pointer_recover_from_lost_button_count, LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(pointer_scroll_wheel, LITEST_WHEEL, LITEST_TABLET);
+	litest_add(pointer_scroll_button, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_scroll_button_noscroll, LITEST_ABSOLUTE|LITEST_BUTTON, LITEST_RELATIVE);
+	litest_add(pointer_scroll_button_noscroll, LITEST_ANY, LITEST_RELATIVE|LITEST_BUTTON);
+	litest_add(pointer_scroll_button_no_event_before_timeout, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_scroll_button_middle_emulation, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_scroll_button_device_remove_while_down, LITEST_ANY, LITEST_RELATIVE|LITEST_BUTTON);
 
-	litest_add("pointer:calibration", pointer_no_calibration, LITEST_ANY, LITEST_TOUCH|LITEST_SINGLE_TOUCH|LITEST_ABSOLUTE|LITEST_PROTOCOL_A|LITEST_TABLET);
+	litest_add(pointer_scroll_button_lock, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_scroll_button_lock_defaults, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_scroll_button_lock_config, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_scroll_button_lock_enable_while_down, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_scroll_button_lock_enable_while_down_just_lock, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_scroll_button_lock_otherbutton, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_scroll_button_lock_enable_while_otherbutton_down, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add_ranged(pointer_scroll_button_lock_middlebutton, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY, &buttonorder);
+	litest_add(pointer_scroll_button_lock_doubleclick_nomove, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+
+	litest_add(pointer_scroll_nowheel_defaults, LITEST_RELATIVE|LITEST_BUTTON, LITEST_WHEEL);
+	litest_add_for_device(pointer_scroll_defaults_logitech_marble , LITEST_LOGITECH_TRACKBALL);
+	litest_add(pointer_scroll_natural_defaults, LITEST_WHEEL, LITEST_TABLET);
+	litest_add(pointer_scroll_natural_defaults_noscroll, LITEST_ANY, LITEST_WHEEL);
+	litest_add(pointer_scroll_natural_enable_config, LITEST_WHEEL, LITEST_TABLET);
+	litest_add(pointer_scroll_natural_wheel, LITEST_WHEEL, LITEST_TABLET);
+	litest_add(pointer_scroll_has_axis_invalid, LITEST_WHEEL, LITEST_TABLET);
+
+	litest_add(pointer_no_calibration, LITEST_ANY, LITEST_TOUCH|LITEST_SINGLE_TOUCH|LITEST_ABSOLUTE|LITEST_PROTOCOL_A|LITEST_TABLET);
 
 									/* tests touchpads too */
-	litest_add("pointer:left-handed", pointer_left_handed_defaults, LITEST_BUTTON, LITEST_ANY);
-	litest_add("pointer:left-handed", pointer_left_handed, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
-	litest_add("pointer:left-handed", pointer_left_handed_during_click, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
-	litest_add("pointer:left-handed", pointer_left_handed_during_click_multiple_buttons, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_left_handed_defaults, LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_left_handed, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_left_handed_during_click, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
+	litest_add(pointer_left_handed_during_click_multiple_buttons, LITEST_RELATIVE|LITEST_BUTTON, LITEST_ANY);
 
-	litest_add("pointer:accel", pointer_accel_defaults, LITEST_RELATIVE, LITEST_ANY);
-	litest_add("pointer:accel", pointer_accel_invalid, LITEST_RELATIVE, LITEST_ANY);
-	litest_add("pointer:accel", pointer_accel_defaults_absolute, LITEST_ABSOLUTE, LITEST_RELATIVE);
-	litest_add("pointer:accel", pointer_accel_defaults_absolute_relative, LITEST_ABSOLUTE|LITEST_RELATIVE, LITEST_ANY);
-	litest_add("pointer:accel", pointer_accel_direction_change, LITEST_RELATIVE, LITEST_POINTINGSTICK);
-	litest_add("pointer:accel", pointer_accel_profile_defaults, LITEST_RELATIVE, LITEST_TOUCHPAD);
-	litest_add("pointer:accel", pointer_accel_profile_defaults_noprofile, LITEST_TOUCHPAD, LITEST_ANY);
-	litest_add("pointer:accel", pointer_accel_profile_invalid, LITEST_RELATIVE, LITEST_ANY);
-	litest_add("pointer:accel", pointer_accel_profile_noaccel, LITEST_ANY, LITEST_TOUCHPAD|LITEST_RELATIVE|LITEST_TABLET);
-	litest_add("pointer:accel", pointer_accel_profile_flat_motion_relative, LITEST_RELATIVE, LITEST_TOUCHPAD);
+	litest_add(pointer_accel_defaults, LITEST_RELATIVE, LITEST_ANY);
+	litest_add(pointer_accel_invalid, LITEST_RELATIVE, LITEST_ANY);
+	litest_add(pointer_accel_defaults_absolute, LITEST_ABSOLUTE, LITEST_RELATIVE);
+	litest_add(pointer_accel_defaults_absolute_relative, LITEST_ABSOLUTE|LITEST_RELATIVE, LITEST_ANY);
+	litest_add(pointer_accel_direction_change, LITEST_RELATIVE, LITEST_POINTINGSTICK);
+	litest_add(pointer_accel_profile_defaults, LITEST_RELATIVE, LITEST_TOUCHPAD);
+	litest_add(pointer_accel_profile_defaults, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add(pointer_accel_profile_invalid, LITEST_RELATIVE, LITEST_ANY);
+	litest_add(pointer_accel_profile_noaccel, LITEST_ANY, LITEST_TOUCHPAD|LITEST_RELATIVE|LITEST_TABLET);
+	litest_add(pointer_accel_profile_flat_motion_relative, LITEST_RELATIVE, LITEST_TOUCHPAD);
 
-	litest_add("pointer:middlebutton", middlebutton, LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add("pointer:middlebutton", middlebutton_nostart_while_down, LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add("pointer:middlebutton", middlebutton_timeout, LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add("pointer:middlebutton", middlebutton_doubleclick, LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add("pointer:middlebutton", middlebutton_middleclick, LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add("pointer:middlebutton", middlebutton_middleclick_during, LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add("pointer:middlebutton", middlebutton_default_enabled, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_POINTINGSTICK);
-	litest_add("pointer:middlebutton", middlebutton_default_clickpad, LITEST_CLICKPAD, LITEST_ANY);
-	litest_add("pointer:middlebutton", middlebutton_default_touchpad, LITEST_TOUCHPAD, LITEST_CLICKPAD);
-	litest_add("pointer:middlebutton", middlebutton_default_disabled, LITEST_ANY, LITEST_BUTTON);
-	litest_add_for_device("pointer:middlebutton", middlebutton_default_alps, LITEST_ALPS_SEMI_MT);
-	litest_add("pointer:middlebutton", middlebutton_button_scrolling, LITEST_RELATIVE|LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add("pointer:middlebutton", middlebutton_button_scrolling_middle, LITEST_RELATIVE|LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add("pointer:middlebutton", middlebutton_device_remove_while_down, LITEST_BUTTON, LITEST_CLICKPAD);
-	litest_add("pointer:middlebutton", middlebutton_device_remove_while_one_is_down, LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(middlebutton, LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(middlebutton_nostart_while_down, LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(middlebutton_timeout, LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(middlebutton_doubleclick, LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(middlebutton_middleclick, LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(middlebutton_middleclick_during, LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(middlebutton_default_enabled, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_POINTINGSTICK);
+	litest_add(middlebutton_default_clickpad, LITEST_CLICKPAD, LITEST_ANY);
+	litest_add(middlebutton_default_touchpad, LITEST_TOUCHPAD, LITEST_CLICKPAD);
+	litest_add(middlebutton_default_disabled, LITEST_ANY, LITEST_BUTTON);
+	litest_add_for_device(middlebutton_default_alps, LITEST_ALPS_SEMI_MT);
+	litest_add(middlebutton_button_scrolling, LITEST_RELATIVE|LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(middlebutton_button_scrolling_middle, LITEST_RELATIVE|LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(middlebutton_device_remove_while_down, LITEST_BUTTON, LITEST_CLICKPAD);
+	litest_add(middlebutton_device_remove_while_one_is_down, LITEST_BUTTON, LITEST_CLICKPAD);
 
-	litest_add_ranged("pointer:state", pointer_absolute_initial_state, LITEST_ABSOLUTE, LITEST_ANY, &axis_range);
+	litest_add_ranged(pointer_absolute_initial_state, LITEST_ABSOLUTE, LITEST_ANY, &axis_range);
 
-	litest_add("pointer:time", pointer_time_usec, LITEST_RELATIVE, LITEST_ANY);
+	litest_add(pointer_time_usec, LITEST_RELATIVE, LITEST_ANY);
 
-	litest_add_ranged("pointer:debounce", debounce_bounce, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE, &buttons);
-	litest_add("pointer:debounce", debounce_bounce_check_immediate, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE);
-	litest_add_ranged("pointer:debounce", debounce_spurious, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE, &buttons);
-	litest_add("pointer:debounce", debounce_spurious_multibounce, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE);
-	litest_add("pointer:debounce_otherbutton", debounce_spurious_dont_enable_on_otherbutton, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE);
-	litest_add("pointer:debounce_otherbutton", debounce_spurious_cancel_debounce_otherbutton, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE);
-	litest_add("pointer:debounce_otherbutton", debounce_spurious_switch_to_otherbutton, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE);
-	litest_add_no_device("pointer:debounce", debounce_remove_device_button_down);
-	litest_add_no_device("pointer:debounce", debounce_remove_device_button_up);
+	litest_add_ranged(debounce_bounce, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE, &buttons);
+	litest_add(debounce_bounce_check_immediate, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE);
+	litest_add_ranged(debounce_spurious, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE, &buttons);
+	litest_add(debounce_spurious_multibounce, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE);
+	litest_add(debounce_spurious_dont_enable_on_otherbutton, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE);
+	litest_add(debounce_spurious_cancel_debounce_otherbutton, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE);
+	litest_add(debounce_spurious_switch_to_otherbutton, LITEST_BUTTON, LITEST_TOUCHPAD|LITEST_NO_DEBOUNCE);
+	litest_add_no_device(debounce_remove_device_button_down);
+	litest_add_no_device(debounce_remove_device_button_up);
 }
